%% mhtmighaz.m
% This script computes and plots inverse Gaussian hazard rates

%% Initialization
disp('[mhtmighaz] Running mhtmighaz.m ...');

%% Set plot options
% Set upper limit of time axis (maxT), step size (step), and the legend x coordinate (legx) and y coordinate offset (dlegy) to defaults if 'iGopt' does not exist 
if ~exist('iGopt','var')
    disp('[mhtmighaz] iG plot options set to defaults');
    iGopt.maxT  = 1; 
    iGopt.step  = 1/1000;
    iGopt.legx  = 0.3*iGopt.maxT;
    iGopt.dlegy = 1/25
end

%% Set thresholds and export to TeX file
% Set to defaults and export if 'threshold' does not exist
if ~exist('threshold','var')
    disp('[mhtmighaz] thresholds set to defaults and exported');
    threshold = [0.3; 0.8; 1.3]
    f1=fopen('mhtmConstants.tex','w');
        fprintf(f1,'\\def\\ylow{%3.1f}\n',threshold(1));    
        fprintf(f1,'\\def\\ymid{%3.1f}\n',threshold(2));
        fprintf(f1,'\\def\\yhigh{%3.1f}\n',threshold(3));
    fclose(f1);
end

%% Set parameters Levy process
% Set to Brownian motion with unit drift and (implicitly) variance if 'levy' does not exist (note that only 'mu' is used in this script)
if ~exist('levy','var')
    disp('[mhtmighaz] Levy process parameters set to defaults');
    levy.BM     = 1;         % Brownian motion on (1) or off (0) 
	levy.mu     = 1;         % drift Brownian motion 
    % Parameters compound Poisson process
    levy.CP     = 0; 		% compound Poisson on (1) or off (0) 
	levy.lambda = 1;         % Poisson arrival rate exponential shocks 
	levy.mshock	= 1 		% mean of shock 
end


%% Compute and plot hazard paths
t           =  (0:iGopt.step:iGopt.maxT)';
nt          = length(t);
iGt         = t(2:nt);
threshmat   = repmat(threshold',nt-1,1);
iGtmat      = repmat(iGt,1,length(threshold));
sriGtmat    = repmat(sqrt(iGt),1,length(threshold));
iGpdf       = threshmat.*normpdf((threshmat-levy.mu*iGtmat)./sriGtmat,0,1)./(sriGtmat.*iGtmat);		
iGsurv      = normcdf((threshmat-levy.mu*iGtmat)./sriGtmat,0,1) - exp(2*levy.mu*threshmat).*normcdf((-threshmat-levy.mu*iGtmat)./sriGtmat,0,1);
iGhazard    = [zeros(1,length(threshold));(iGpdf./iGsurv)];

plot(t,iGhazard);


%% Export hazard path data to csv file and auxiliary parameters to TeX fike for TikZ plotting

f1=fopen('mhtmighaz.csv','w');
fprintf(f1,'t, h1, h2, h3, dummy\n');
fprintf(f1,'%6.6f, %6.6f, %6.6f, %6.6f, 1.0\n',[t iGhazard]');
fclose(f1);

hmin = floor(min(min(iGhazard)));
hmax = max(max(iGhazard));
legind = find(t>=iGopt.legx, 1, 'first');

f1=fopen('mhtmighazConstants.tex','w');
    % Levy parameters
    fprintf(f1,'\\def\\iGmu{%3.0f}\n',levy.mu);
    fprintf(f1,'\\def\\iGsigma{%3.0f}\n',1);
    % Plot parameters
    fprintf(f1,'\\def\\iGtmin{%3.0f}\n',floor(min(t)));
    fprintf(f1,'\\def\\iGtmax{%3.0f}\n',ceil(max(t)));
    fprintf(f1,'\\def\\iGhmin{%3.0f}\n',hmin);
    fprintf(f1,'\\def\\iGhmax{%3.2f}\n',hmax);
    fprintf(f1,'\\def\\iGlegx{%6.6f}\n',iGopt.legx);
    fprintf(f1,'\\def\\iGlegylow{%6.6f}\n',iGhazard(legind,1)+2.5*(hmax-hmin)*iGopt.dlegy);
    fprintf(f1,'\\def\\iGlegymid{%6.6f}\n',iGhazard(legind,2)+(hmax-hmin)*iGopt.dlegy);
    fprintf(f1,'\\def\\iGlegyhigh{%6.6f}\n',iGhazard(legind,3)+(hmax-hmin)*iGopt.dlegy);
fclose(f1);
