%% mhtmsim.m
% This script simulates and plots Levy process sample paths.

%% Initialization
disp('[mhtmsim] Running mhtmsim.m ...');

%% Initialize pseudo random number generator
% Set seed
randn('seed',23061931);

%% Set plot options
% Set upper limit of time axis (maxT), step size (step), the legend x coordinate (legx) and y coordinate offset (dlegy), and the number of simulated paths (nsim) to defaults if 'simopt' does not exist 
if ~exist('simopt','var')
    disp('[mhtmsim] simulation plot options set to defaults');
    simopt.maxT  = 1; 
    simopt.step  = 1/2000;
    simopt.legx  = 0.78*simopt.maxT;
    simopt.dlegy = 1/25
    simopt.nsim  =2 				    
end

%% Set thresholds and export to TeX file
% Set to defaults and export if 'threshold' does not exist
if ~exist('threshold','var')
    disp('[mhtmsim] thresholds set to defaults and exported');
    threshold = [0.3; 0.8; 1.3]
    f1=fopen('mhtmConstants.tex','w');
        fprintf(f1,'\\def\\ylow{%3.1f}\n',threshold(1));    
        fprintf(f1,'\\def\\ymid{%3.1f}\n',threshold(2));
        fprintf(f1,'\\def\\yhigh{%3.1f}\n',threshold(3));
    fclose(f1);
end
nthresh = length(threshold);

%% Set parameters Levy process
% Set to Brownian motion with unit drift and (implicitly) variance if 'levy' does not exist
if ~exist('levy','var')
    disp('[mhtmsim] Levy process parameters set to defaults');
    levy.BM     = 1;         % Brownian motion on (1) or off (0) 
	levy.mu     = 1;         % drift Brownian motion 
    % Parameters compound Poisson process
    levy.CP     = 0; 		% compound Poisson on (1) or off (0) 
	levy.lambda = 1;         % Poisson arrival rate exponential shocks 
	levy.mshock	= 1 		% mean of shock 
end


%% Simulate and plot sample paths
t  	=  (0:simopt.step:simopt.maxT)';
nt  = length(t);
if levy.CP
	negshock = (rand(nt-1,simopt.nsim)<=levy.lambda*simopt.step);
	shock	= levy.mshock*negshock.*ln(rand(nt-1,simopt.nsim));
else
        shock = 0; 
end
if levy.BM
	dnorm	= randn(nt-1,simopt.nsim)*sqrt(simopt.step);
else
    dnorm = 0; 
end

samplepaths	= [zeros(1,simopt.nsim);cumsum(dnorm+shock)]+ levy.mu*t*ones(1,simopt.nsim);

plot(t,[repmat(threshold',nt,1) samplepaths]);

%% Compute hitting times


%% Export sample path data to csv file and auxiliary parameters to TeX fike for TikZ plotting
% Allow only two paths, and compute hitting times for each threshold
if simopt.nsim ~= 2
        disp('[mhtmsim] number of similations reset to 2');
        simopt.nsim = 2
        samplepaths = samplepaths(:,1:2);
end
f1=fopen('mhtmsim.csv','w');
    fprintf(f1,'t, p1, p2, dummy\n');
    fprintf(f1,'%6.6f, %6.6f, %6.6f, 1.0\n',[t samplepaths]');
fclose(f1);

hmin = min(min(samplepaths));
hmax = max(max(samplepaths));
tmin = floor(min(t));
tmax = ceil(max(t));
legind = find(t>=simopt.legx, 1, 'first');
f1=fopen('mhtmsimConstants.tex','w');
    % Levy parameters   
    fprintf(f1,'\\def\\simBM{%3.0f}\n',levy.BM);
    fprintf(f1,'\\def\\simmu{%3.0f}\n',levy.mu);
    fprintf(f1,'\\def\\simsigma{%3.0f}\n',1);
    fprintf(f1,'\\def\\simCP{%3.0f}\n',levy.CP);
    fprintf(f1,'\\def\\simlambda{%3.2f}\n',levy.lambda);
    fprintf(f1,'\\def\\simmshock{%3.2f}\n',levy.mshock);
    % Plot parameters
    fprintf(f1,'\\def\\simtmin{%3.0f}\n',tmin);
    fprintf(f1,'\\def\\simtmax{%3.0f}\n',tmax);
    fprintf(f1,'\\def\\simhmin{%3.2f}\n',hmin);
    fprintf(f1,'\\def\\simhmax{%3.2f}\n',hmax);
    fprintf(f1,'\\def\\simlegx{%6.6f}\n',simopt.legx);
    for sim=1:simopt.nsim
        fprintf(f1,['\\def\\simlegy' char(96+sim) '{%6.6f}\n'],samplepaths(legind,sim)+(hmin-hmax)*simopt.dlegy);
    end
    % Hitting times
    for sim=1:simopt.nsim
        for thr=1:nthresh
        fprintf(f1,['\\def\\simht' char(96+sim) char(96+thr) '{%6.6f}\n'],t(find(samplepaths(:,sim)>threshold(thr),1,'first')));
        fprintf(f1,['\\def\\simhtx' char(96+sim) char(96+thr) '{%3.2f}\n'],t(find(samplepaths(:,sim)>threshold(thr),1,'first')));
        end
    end
    
fclose(f1);
